@@ -35,6 +35,7 @@ CACHE_KEYS = {}
 SQLS = {}
 MEMORY_FILE = ':memory:'
 cur_filename = MEMORY_FILE
+cur_tablename = 'kudb'
 SQLITE_MAX_INT=9223372036854775807
 # SQL template
 SQLS_TEMPLATE = {
@@ -72,7 +73,8 @@ SQLS_TEMPLATE = {
     'get_doc_by_id': 'SELECT value, id FROM doc__TABLE_NAME__ WHERE id=?',
     'get_doc_by_tag': 'SELECT value, id FROM doc__TABLE_NAME__ WHERE tag=? LIMIT ?',
     'insert_doc': 'INSERT INTO doc__TABLE_NAME__ (value, tag, ctime, mtime) VALUES (?, ?, ?, ?)',
-    'update_doc': 'UPDATE doc__TABLE_NAME__ SET value=?, mtime=? WHERE id=?',
+    'update_doc': 'UPDATE doc__TABLE_NAME__ SET value=?, tag=?, mtime=? WHERE id=?',
+    'update_doc_by_tag': 'UPDATE doc__TABLE_NAME__ SET value=?, tag=?, mtime=? WHERE tag=?',
     'delete_doc': 'DELETE FROM doc__TABLE_NAME__ WHERE id=?',
     'delete_doc_by_tag': 'DELETE FROM doc__TABLE_NAME__ WHERE tag=?',
     'clear_doc': 'DELETE FROM doc__TABLE_NAME__',
@@ -81,13 +83,13 @@ SQLS_TEMPLATE = {
 
 def connect(filename = ':memory:', table_name='kudb'):
     """Connect to database"""
-    global SQLS, cur_filename, db
+    global SQLS, cur_filename, db, cur_tablename
     # generate sqls
     SQLS = {}
     for key, val in SQLS_TEMPLATE.items():
         SQLS[key] = val.replace('__TABLE_NAME__', table_name)
     # check cache
-    if filename in cache_db: # already open?
+    if (filename in cache_db) and (cur_tablename == table_name): # already open?
         db = cache_db[filename] # use_cache
         cur_filename = filename
         return db
@@ -95,6 +97,7 @@ def connect(filename = ':memory:', table_name='kudb'):
     db = sqlite3.connect(filename, check_same_thread=False)
     cache_db[filename] = db
     cur_filename = filename
+    cur_tablename = table_name
     try:
         # create table
         db.executescript(SQLS['create'] + ';' + SQLS['create_doc'])
@@ -144,11 +147,14 @@ def get_key(key, default = '', file=None):
         return default
     cur = db.cursor()
     try:
-        cur.execute(SQLS['select'], [key])
+        sql = SQLS['select']
+        cur.execute(sql, [key])
         values = cur.fetchone()
+        if values is None:
+            return default
         return json.loads(values[0])
     except Exception as err:
-        raise Exception('could not read database: ' + str(err)) from err
+        raise Exception(f'`get_key(%s)` could not read database: %s' % key, str(err)) from err
     finally:
         cur.close()
 
@@ -171,27 +177,31 @@ def get_info(key, default = ''):
 def set_key(key, value, file=None):
     """
     set data by key
-    >>> set_key('hoge', 30, file=':memory:')
+    >>> set_key('hoge', 30, file=':memory:') # insert
     >>> get_key('hoge')
     30
     >>> set_key(1, 40)
     >>> get_key(1)
     40
+    >>> set_key('hoge', 35) # update
+    >>> get_key('hoge')
+    35
     """
     if file is not None:
         connect(file)
     if db is None:
         raise Exception('please connect before using `set_key` method.')
     try:
+        value_json = json.dumps(value, ensure_ascii=False)
         cur = db.cursor()
         if key in CACHE_KEYS:
-            cur.execute(SQLS['update'], [value, int(time.time()), key])
+            cur.execute(SQLS['update'], [value_json, int(time.time()), key])
         else:
             cur.execute(
                 SQLS['insert'],
                 [
                     key,
-                    json.dumps(value, ensure_ascii=False),
+                    value_json,
                     int(time.time()),
                     int(time.time())
                 ]
@@ -493,6 +503,8 @@ def insert_many(value_list, file=None, tag=None):
     # check tag
     if tag is None:
         tag = get_key('_tag', 'tag')
+    else:
+        set_key('_tag', tag)
     rows = []
     for val in value_list:
         tag_value = ''
@@ -509,9 +521,11 @@ def insert_many(value_list, file=None, tag=None):
     except Exception as err:
         raise Exception('database insert error:' + str(err)) from err
 
-def update(doc_id, values):
+def update(id=None, new_value=None, tag=None):
     """
     update doc
+
+    update by id
     >>> clear(file=MEMORY_FILE)
     >>> insert_many([1,2,3,4,5])
     >>> get_by_id(1)
@@ -519,17 +533,66 @@ def update(doc_id, values):
     >>> update(1, 100)
     >>> get_by_id(1)
     100
+
+    update by id:
+    >>> clear()
+    >>> insert_many([{"name": "A", "age": 30}, {"name": "B", "age": 20}], tag="name")
+    >>> update(id=2, new_value={"name":"B", "age": 10})
+    >>> get_by_tag("B")[0]["age"]
+    10
+
+    update by tag:
+    >>> clear()
+    >>> insert_many([{"name": "A", "age": 30}, {"name": "B", "age": 20}], tag="name")
+    >>> update(tag="B", new_value={"name":"B", "age": 15})
+    >>> get_by_tag("B")[0]["age"]
+    15
     """
     if db is None:
         raise Exception('please connect before using `update_doc` method.')
+    # check tag
+    tag_name = get_key('_tag', 'tag')
+    tag_value = ''
+    if isinstance(new_value, dict):
+        if tag_name in new_value:
+            tag_value = str(new_value[tag_name])
+    # update
     try:
         cur = db.cursor()
-        cur.execute(SQLS['update_doc'], [json.dumps(values), int(time.time()), doc_id])
+        sql = ''
+        if id is not None:
+            sql = SQLS['update_doc']
+            cur.execute(sql, [json.dumps(new_value, ensure_ascii=False), tag_value, int(time.time()), id])
+        elif tag is not None:
+            sql = SQLS['update_doc_by_tag']
+            cur.execute(sql, [json.dumps(new_value, ensure_ascii=False), tag_value, int(time.time()), tag])
         cur.close()
         db.commit()
     except Exception as err:
         raise Exception('database update error:' + str(err)) from err
 
+def update_by_tag(tag, new_value):
+    """
+    update doc value by tag
+    >>> clear()
+    >>> insert_many([{"name": "A", "age": 30}, {"name": "B", "age": 20}], tag="name")
+    >>> update_by_tag("B", {"name":"B", "age": 15})
+    >>> get_by_tag("B")[0]["age"]
+    15
+    """
+    update(tag=tag, new_value=new_value)
+
+def update_by_id(id, new_value):
+    """
+    update doc value by tag
+    >>> clear()
+    >>> insert_many([{"name": "A", "age": 30}, {"name": "B", "age": 20}], tag="name")
+    >>> update_by_tag("B", {"name":"B", "age": 15})
+    >>> get_by_tag("B")[0]["age"]
+    15
+    """
+    update(id=id, new_value=new_value)
+
 def delete(id=None, key=None, tag=None, file=None):
     """
     delete by id or key
